"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const extendWithFirebaseConfig_1 = __importDefault(require("./extendWithFirebaseConfig"));
const tasks = __importStar(require("./tasks"));
const firebase_utils_1 = require("./firebase-utils");
/**
 * Cypress plugin which attaches tasks used by custom commands
 * and returns modified Cypress config. Modified config includes
 * env setting with values of Firebase specific environment variables
 * such as GCLOUD_PROJECT, FIREBASE_DATABASE_EMULATOR_HOST,
 * FIRESTORE_EMULATOR_HOST and FIREBASE_AUTH_EMULATOR_HOST.
 * @param cypressOnFunc - on function from cypress plugins file
 * @param cypressConfig - Cypress config
 * @param adminInstance - firebase-admin instance
 * @param overrideConfig - Override config for firebase instance
 * @returns Extended Cypress config
 */
function pluginWithTasks(cypressOnFunc, cypressConfig, adminInstance, overrideConfig) {
    // Only initialize admin instance if it hasn't already been initialized
    if (adminInstance.apps && adminInstance.apps.length === 0) {
        (0, firebase_utils_1.initializeFirebase)(adminInstance, overrideConfig);
    }
    const tasksWithFirebase = Object.keys(tasks).reduce((acc, taskName) => {
        acc[taskName] = (taskSettings) => {
            if (taskSettings && taskSettings.uid) {
                return tasks[taskName](adminInstance, taskSettings.uid, taskSettings);
            }
            const { action, path: actionPath, options = {}, data } = taskSettings;
            return tasks[taskName](adminInstance, action, actionPath, options, data);
        };
        return acc;
    }, {});
    // Attach tasks to Cypress using on function
    cypressOnFunc('task', tasksWithFirebase);
    // Return extended config
    return (0, extendWithFirebaseConfig_1.default)(cypressConfig);
}
exports.default = pluginWithTasks;
